
import os
import base64
import logging

import pyaas

def generateCookieSecret(path):
    secret = base64.b64encode(os.urandom(32))
    with open(path, 'w') as fp:
        fp.write(secret)
    return secret




import tornado.auth

#
# Google authentication
#

_domains = []


class Login(tornado.web.RequestHandler, tornado.auth.GoogleMixin):

    @classmethod
    def initialize(cls, *args, **kwargs):
        _domains.extend([d for d in kwargs.get('domains', '').split(' ') if d])

    @tornado.web.asynchronous
    def get(self):
        if self.get_argument('openid.mode', None):
            self.get_authenticated_user(self._on_auth)
            return
        self.authenticate_redirect()

    def _on_auth(self, user):
        if not user:
            raise tornado.web.HTTPError(500, 'Google authentication failed')

        try:
            name, domain = user['email'].split('@', 1)
        except:
            raise tornado.web.HTTPError(500, 'Google authentication failed')

        if _domains and domain not in _domains:
            raise tornado.web.HTTPError(404, 'Invalid domain')

        self.set_secure_cookie('uid', name)

        self.redirect(self.get_argument('next', '/'))

    def write_error(self, status_code, **kwargs):
        #''.join("<strong>%s</strong>: %s<br/>" % (k, self.request.__dict__[k]) for k in self.request.__dict__.keys())
        #if 'exc_info' not in kwargs:
        #    return

        e = kwargs['exc_info'][1]

        self.set_header('Content-Type', 'text/html')
        self.finish('''
            <html>
                <head><title>%s</title></head>
                <body><p>%s</p></body>
            </html>''' % (status_code, e.log_message))




import os
import hashlib
import base64

import pyaas

import tornado.web

# TODO: this is opening the file and scanning it for every login, which is fine
#       for small projects with a limited number of users.  Long term this
#       this should be updated to support faster lookups

# Passwords can be managed with Apache's htpasswd program

class Login(tornado.web.RequestHandler):
    def get(self):
        self.render('login.html')

    def post(self):
        username = self.get_argument('username', 'admin')
        password = self.get_argument('password', '')
        password = '{SHA}' + base64.b64encode(hashlib.sha1(password).digest())

        path = os.path.join(pyaas.prefix, pyaas.config.get('basic', 'path'))
        fp = open(path, 'r')
        for line in fp:
            if not line:
                continue
            entry,hash = line.split(':', 1)

            if entry != username:
                continue

            if hash.rstrip() != password:
                break
                #raise tornado.web.HTTPError(403)

            self.set_secure_cookie('uid', username)
            break

        self.redirect(self.get_argument('next', '/'))




import pyaas.web

class ExampleApp(pyaas.web.Application):
    def __init__(self):
        # important to use extend if using built-in authentication
        self.patterns = [
            ( r'/',       pyaas.web.handlers.Index     ),
            ( r'/(main)', pyaas.web.handlers.Protected ),
            ]

        pyaas.web.Application.__init__(self)